import { Component, OnInit } from '@angular/core';
import { ActivatedRoute, ParamMap, Router } from '@angular/router';
import { Organization } from 'src/app/models/organization.model';
import { OrganizationService } from 'src/app/services/organization.service';
import { FormBuilder, FormGroup, Validators, FormArray } from '@angular/forms';
import { filter, switchMap, mergeMap } from 'rxjs/operators';
import { IImageUploadFieldParams } from 'src/app/models/image-upload.model';
import { NotificationService } from 'src/app/services';

@Component({
  selector: 'app-organization-form',
  templateUrl: './organization-form.component.html',
  styleUrls: ['./organization-form.component.scss'],
})
export class OrganizationFormComponent implements OnInit {

  organization: Organization;
  form: FormGroup;
  logoFile: File;
  logoFieldParams: IImageUploadFieldParams = {
    inputName: 'logo',
    label: 'Logo',
    existingImageUrl: null,
  };
  logo: File;
  title: string;

  constructor(
    private _organizationService: OrganizationService,
    private _notificationService: NotificationService,
    private _route: ActivatedRoute,
    private _router: Router,
    private _fb: FormBuilder,
  ) {
  }

  ngOnInit() {
    this.title = this._route.snapshot.data.title;
    this.initForm();

    this._route.paramMap.pipe(
      filter((paramMap: ParamMap) => (paramMap.get('id') !== null)),
      switchMap((paramMap: ParamMap) => this._organizationService.findById(paramMap.get('id'))))
      .subscribe((organization: Organization) => {

        this.organization = organization;

        const arr = new FormArray([]);
        this.organization.links.forEach((y) => {
          arr.push(this._fb.group({
            url: y.url,
            organizationId: y.organizationId,
          }));
        });

        this.logoFieldParams.existingImageUrl = organization.logo;

        this.form = this._fb.group(
          {
            id: [this.organization.id],
            name: [organization.name, Validators.required],
            description: [organization.description, Validators.required],
            elasticSearchName: [organization.elasticSearchName],
            logo: [organization.logo],
            links: arr,
          });
      });

  }

  initForm() {
    this.organization = new Organization();
    const arr = new FormArray([]);
    this.logoFieldParams.existingImageUrl = null;
    this.form = this._fb.group(
      {
        id: [this.organization.id],
        name: [this.organization.name, Validators.required],
        description: [this.organization.description, Validators.required],
        elasticSearchName: [this.organization.elasticSearchName],
        logo: [this.organization.logo],
        links: arr,
      });
  }

  initItemLinkForm() {
    return this._fb.group({
      name: '',
      url: ['', Validators.required],
    });
  }

  addLink() {
    if (!this.form.controls.links) {
      this.organization.links = [];
    }
    (this.form.controls.links as FormArray).push(this._fb.group(
      {
        id: null, name: '',
        url: ['', Validators.required],
        organizationId: this.organization.id,
      }),
    );
  }

  removeLink(index) {
    (this.form.controls.links as FormArray).removeAt(index);
  }

  get formLinks() {
    return this.form.controls.links as FormArray;
  }

  onSubmit() {
    if (!this.formInvalid) {
      this.organization = new Organization(this.form.value);
      let action;

      if (this.logoFile) {
        action = this._organizationService.uploadLogoAndSaveOrganization(this.logoFile, this.organization);
      } else {
        action = this._organizationService.replaceOrCreate(this.organization);
      }

      action.subscribe(
        (organizationCreated) => {
          this._notificationService.notify({
            message: 'L\'organisation a été créée avec succès.',
            type: 'success',
          });
          this._router.navigate(['/organizations', organizationCreated.id]);
        },
        () => {
          this._notificationService.notify({
            message: 'Une erreur est survenue lors de la création de l\'organisation.',
            type: 'error',
          });
        },
      );
    }
  }

  // Getters for each property
  get name() {
    return this.form.controls['name'];
  }

  get description() {
    return this.form.controls['description'];
  }

  get formInvalid() {
    return this.form.invalid ||
      (!this.organization.logo && !this.logoFile);
  }

  logoChanged(fileList: FileList) {
    if (fileList && fileList.length > 0) {
      this.logoFile = fileList[0];
    }
  }

  removeLogo() {
    this.form.get('logo').setValue(null);
  }
}
